import os
from dotenv import load_dotenv
from typing import List, Dict, Any, Tuple, Optional
import json
import time
import asyncio
import gradio as gr
from swarms.structs.agent import Agent
from swarms.structs.swarm_router import SwarmRouter
from swarms.utils.loguru_logger import initialize_logger
import re
import csv # Import the csv module for csv parsing
from swarms.utils.litellm_wrapper import LiteLLM
from litellm import models_by_provider
from dotenv import set_key, find_dotenv
# Initialize logger
logger = initialize_logger(log_folder="swarm_ui")

# Load environment variables
load_dotenv()

# Get the OpenAI API key from the environment variable
api_key = os.getenv("GROQ_API_KEY")

# Define the path to agent_prompts.json
# locates the json file and then fetches the promopts

PROMPT_JSON_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), "agent_prompts.json")
logger.info(f"Loading prompts from: {PROMPT_JSON_PATH}")

def load_prompts_from_json() -> Dict[str, str]:
    try:
        if not os.path.exists(PROMPT_JSON_PATH):
            # Load default prompts
            return {
                "agent.data_extractor": "You are a data extraction agent...",
                "agent.summarizer": "You are a summarization agent...",
                "agent.onboarding_agent": "You are an onboarding agent..."
            }

        with open(PROMPT_JSON_PATH, 'r', encoding='utf-8') as f:
            try:
                data = json.load(f)
            except json.JSONDecodeError:
                # Load default prompts
                return {
                    "agent.data_extractor": "You are a data extraction agent...",
                    "agent.summarizer": "You are a summarization agent...",
                    "agent.onboarding_agent": "You are an onboarding agent..."
                }

            if not isinstance(data, dict):
                # Load default prompts
                 return {
                    "agent.data_extractor": "You are a data extraction agent...",
                    "agent.summarizer": "You are a summarization agent...",
                    "agent.onboarding_agent": "You are an onboarding agent..."
                }

            prompts = {}
            for agent_name, details in data.items():
                if not isinstance(details, dict) or "system_prompt" not in details:
                   continue

                prompts[f"agent.{agent_name}"] = details["system_prompt"]

            if not prompts:
               # Load default prompts
                return {
                    "agent.data_extractor": "You are a data extraction agent...",
                    "agent.summarizer": "You are a summarization agent...",
                    "agent.onboarding_agent": "You are an onboarding agent..."
                }

            return prompts

    except Exception:
        # Load default prompts
        return {
            "agent.data_extractor": "You are a data extraction agent...",
            "agent.summarizer": "You are a summarization agent...",
            "agent.onboarding_agent": "You are an onboarding agent..."
        }

AGENT_PROMPTS = load_prompts_from_json()

def initialize_agents(dynamic_temp: float, agent_keys: List[str]) -> List[Agent]:
    agents = []
    seen_names = set()
    for agent_key in agent_keys:
        if agent_key not in AGENT_PROMPTS:
            raise ValueError(f"Invalid agent key: {agent_key}")

        agent_prompt = AGENT_PROMPTS[agent_key]
        agent_name = agent_key.split('.')[-1]

        # Ensure unique agent names
        base_name = agent_name
        counter = 1
        while agent_name in seen_names:
            agent_name = f"{base_name}_{counter}"
            counter += 1
        seen_names.add(agent_name)

        agent = Agent(
            agent_name=f"Agent-{agent_name}",
            system_prompt=agent_prompt,
            llm=model,
            max_loops=1,
            autosave=True,
            verbose=True,
            dynamic_temperature_enabled=True,
            saved_state_path=f"agent_{agent_name}.json",
            user_name="pe_firm",
            retry_attempts=1,
            context_length=200000,
            output_type="string",  # here is the output type which is string
            temperature=dynamic_temp, # THIS LINE IS CHANGED
        )
        agents.append(agent)

    return agents

async def execute_task(task: str, max_loops: int, dynamic_temp: float,
                        swarm_type: str, agent_keys: List[str], flow: str = None, model_name:str = "gpt-4o",
                         provider:str = "openai", api_key:str=None, temperature:float=0.5, max_tokens:int=4000) -> Tuple[Any, 'SwarmRouter', str]:
    try:
        # Initialize agents
        try:
             agents = initialize_agents(dynamic_temp, agent_keys, model_name, provider, api_key, temperature, max_tokens)
        except Exception as e:
            return None, None, str(e)

        # Swarm-specific configurations
        router_kwargs = {
            "name": "multi-agent-workflow",
            "description": f"Executing {swarm_type} workflow",
            "max_loops": max_loops,
            "agents": agents,
            "autosave": True,
            "return_json": True,
            "output_type": "string",
            "swarm_type": swarm_type,  # Pass swarm_type here
        }

        if swarm_type == "AgentRearrange":
            if not flow:
                return None, None, "Flow configuration is required for AgentRearrange"
            router_kwargs["flow"] = flow

        if swarm_type == "MixtureOfAgents":
            if len(agents) < 2:
                return None, None, "MixtureOfAgents requires at least 2 agents"
        
        if swarm_type == "SequentialWorkflow":
            if len(agents) < 2:
                return None, None, "SequentialWorkflow requires at least 2 agents"


        # Create and execute SwarmRouter
        try:
            timeout = 450 if swarm_type != "SpreadSheetSwarm" else 900  # SpreadSheetSwarm will have different timeout.
            router = SwarmRouter(**router_kwargs)

            if swarm_type == "AgentRearrange":
                 result = await asyncio.wait_for(
                    asyncio.to_thread(router._run, task),
                    timeout=timeout
                )
                 return result, router, ""

            result = await asyncio.wait_for(
                asyncio.to_thread(router.run, task),
                timeout=timeout
            )

            return result, router, ""

        except asyncio.TimeoutError:
            return None, None, f"Task execution timed out after {timeout} seconds"
        except Exception as e:
            return None, None, str(e)

    except Exception as e:
        return None, None, str(e)

def parse_agent_rearrange_output(data: Optional[str]) -> str:
    """
    Parses the AgentRearrange output string and formats it for display.
    """
    if data is None:
        return "Error: No data provided for parsing."

    print(f"Raw data received for parsing:\n{data}")  # Debug: Print raw data

    try:
        parsed_data = json.loads(data)

        if "input" not in parsed_data or not isinstance(parsed_data.get("input"), dict):
            return "Error: 'input' data is missing or not a dictionary."
        
        if "swarm_id" not in parsed_data["input"] :
            return "Error: 'swarm_id' key is missing in the 'input'."
        
        if "name" not in parsed_data["input"]:
           return "Error: 'name' key is missing in the 'input'."
        
        if "flow" not in parsed_data["input"]:
           return "Error: 'flow' key is missing in the 'input'."
        
        if "time" not in parsed_data :
           return "Error: 'time' key is missing."

        swarm_id = parsed_data["input"]["swarm_id"]
        swarm_name = parsed_data["input"]["name"]
        agent_flow = parsed_data["input"]["flow"]
        overall_time = parsed_data["time"]


        output = f"Workflow Execution Details\n\n"
        output += f"Swarm ID: `{swarm_id}`\n"
        output += f"Swarm Name: `{swarm_name}`\n"
        output += f"Agent Flow: `{agent_flow}`\n\n---\n"
        output += f"Agent Task Execution\n\n"

        if "outputs" not in parsed_data:
            return "Error: 'outputs' key is missing"
            
        if  parsed_data["outputs"] is None:
            return "Error: 'outputs' data is None"
            
        if not isinstance(parsed_data["outputs"], list):
            return "Error: 'outputs' data is not a list."

        for i, agent_output in enumerate(parsed_data["outputs"], start=3):
            if not isinstance(agent_output, dict):
                 return f"Error: Agent output at index {i} is not a dictionary"
            
            if "agent_name" not in agent_output:
                return f"Error: 'agent_name' key is missing at index {i}"
            
            if "steps" not in agent_output:
               return f"Error: 'steps' key is missing at index {i}"
            
            if agent_output["steps"] is None:
               return f"Error: 'steps' data is None at index {i}"
            
            if not isinstance(agent_output["steps"], list):
               return f"Error: 'steps' data is not a list at index {i}"

            agent_name = agent_output["agent_name"]
            output += f"Run {(3-i)} (Agent: `{agent_name}`)**\n\n"
            # output += "<details>\n<summary>Show/Hide Agent Steps</summary>\n\n"

            # Iterate over steps
            for j, step in enumerate(agent_output["steps"], start=3):
                 if not isinstance(step, dict):
                     return f"Error: step at index {j} is not a dictionary at {i} agent output."
                 
                 if step is None:
                     return f"Error: step at index {j} is None at {i} agent output"

                 if "role" not in step:
                     return f"Error: 'role' key missing at step {j} at {i} agent output."
                 
                 if "content" not in step:
                    return f"Error: 'content' key missing at step {j} at {i} agent output."
                 
                 if step["role"].strip() != "System:":  # Filter out system prompts
                    #  role = step["role"]
                     content = step["content"]
                     output += f"Step {(3-j)}: \n"
                     output += f"Response :\n {content}\n\n"

            # output += "</details>\n\n---\n"

        output += f"Overall Completion Time: `{overall_time}`"
        return output
    except json.JSONDecodeError as e :
       return f"Error during parsing: json.JSONDecodeError {e}"
    
    except Exception as e:
        return f"Error during parsing: {str(e)}"

def parse_mixture_of_agents_output(data: Optional[str]) -> str:
    """Parses the MixtureOfAgents output string and formats it for display."""
    if data is None:
        return "Error: No data provided for parsing."
    
    print(f"Raw data received for parsing:\n{data}")  # Debug: Print raw data

    try:
        parsed_data = json.loads(data)
        
        if "InputConfig" not in parsed_data or not isinstance(parsed_data["InputConfig"], dict):
            return "Error: 'InputConfig' data is missing or not a dictionary."
        
        if "name" not in parsed_data["InputConfig"]:
           return "Error: 'name' key is missing in 'InputConfig'."
        if "description" not in parsed_data["InputConfig"]:
           return "Error: 'description' key is missing in 'InputConfig'."
        
        if "agents" not in parsed_data["InputConfig"] or not isinstance(parsed_data["InputConfig"]["agents"], list) :
           return "Error: 'agents' key is missing in 'InputConfig' or not a list."
        

        name = parsed_data["InputConfig"]["name"]
        description = parsed_data["InputConfig"]["description"]

        output = f"Mixture of Agents Workflow Details\n\n"
        output += f"Name: `{name}`\n"
        output += f"Description: `{description}`\n\n---\n"
        output += f"Agent Task Execution\n\n"
        
        for agent in parsed_data["InputConfig"]["agents"]:
            if not isinstance(agent, dict):
               return "Error: agent is not a dict in InputConfig agents"
            if "agent_name" not in agent:
                return "Error: 'agent_name' key is missing in agents."
            
            if "system_prompt" not in agent:
                 return f"Error: 'system_prompt' key is missing in agents."

            agent_name = agent["agent_name"]
            # system_prompt = agent["system_prompt"]
            output += f"Agent: `{agent_name}`\n"
            # output += f"*   **System Prompt:** `{system_prompt}`\n\n"


        if "normal_agent_outputs" not in parsed_data or not isinstance(parsed_data["normal_agent_outputs"], list) :
              return "Error: 'normal_agent_outputs' key is missing or not a list."
        
        for i, agent_output in enumerate(parsed_data["normal_agent_outputs"], start=3):
             if not isinstance(agent_output, dict):
                return f"Error: agent output at index {i} is not a dictionary."
             if "agent_name" not in agent_output:
                 return f"Error: 'agent_name' key is missing at index {i}"
             if "steps" not in agent_output:
                return f"Error: 'steps' key is missing at index {i}"
             
             if agent_output["steps"] is None:
                 return f"Error: 'steps' is None at index {i}"
             if not isinstance(agent_output["steps"], list):
                return f"Error: 'steps' data is not a list at index {i}."

             agent_name = agent_output["agent_name"]
             output += f"Run {(3-i)} (Agent: `{agent_name}`)\n\n"
            #  output += "<details>\n<summary>Show/Hide Agent Steps</summary>\n\n"
             for j, step in enumerate(agent_output["steps"], start=3):
                 if not isinstance(step, dict):
                    return f"Error: step at index {j} is not a dictionary at {i} agent output."
                 
                 if step is None:
                     return f"Error: step at index {j} is None at {i} agent output."
                 
                 if "role" not in step:
                    return f"Error: 'role' key missing at step {j} at {i} agent output."

                 if "content" not in step:
                    return f"Error: 'content' key missing at step {j} at {i} agent output."
                
                 if step["role"].strip() != "System:":  # Filter out system prompts
                    #  role = step["role"]
                     content = step["content"]
                     output += f"Step {(3-j)}: \n"
                     output += f"Response:\n {content}\n\n"

            #  output += "</details>\n\n---\n"

        if "aggregator_agent_summary" in parsed_data:
            output += f"\nAggregated Summary :\n{parsed_data['aggregator_agent_summary']}\n{'=' * 50}\n"
        
        return output

    except json.JSONDecodeError as e:
         return f"Error during parsing json.JSONDecodeError : {e}"
    
    except Exception as e:
        return f"Error during parsing: {str(e)}"

def parse_sequential_workflow_output(data: Optional[str]) -> str:
   """Parses the SequentialWorkflow output string and formats it for display."""
   if data is None:
      return "Error: No data provided for parsing."
   
   print(f"Raw data received for parsing:\n{data}")  # Debug: Print raw data

   try:
        parsed_data = json.loads(data)

        if "input" not in parsed_data or not isinstance(parsed_data.get("input"), dict):
            return "Error: 'input' data is missing or not a dictionary."
        
        if "swarm_id" not in parsed_data["input"] :
            return "Error: 'swarm_id' key is missing in the 'input'."
        
        if "name" not in parsed_data["input"]:
           return "Error: 'name' key is missing in the 'input'."
        
        if "flow" not in parsed_data["input"]:
           return "Error: 'flow' key is missing in the 'input'."
        
        if "time" not in parsed_data :
           return "Error: 'time' key is missing."

        swarm_id = parsed_data["input"]["swarm_id"]
        swarm_name = parsed_data["input"]["name"]
        agent_flow = parsed_data["input"]["flow"]
        overall_time = parsed_data["time"]


        output = f"Workflow Execution Details\n\n"
        output += f"Swarm ID: `{swarm_id}`\n"
        output += f"Swarm Name: `{swarm_name}`\n"
        output += f"Agent Flow: `{agent_flow}`\n\n---\n"
        output += f"Agent Task Execution\n\n"

        if "outputs" not in parsed_data:
            return "Error: 'outputs' key is missing"
            
        if  parsed_data["outputs"] is None:
            return "Error: 'outputs' data is None"
            
        if not isinstance(parsed_data["outputs"], list):
            return "Error: 'outputs' data is not a list."

        for i, agent_output in enumerate(parsed_data["outputs"], start=3):
            if not isinstance(agent_output, dict):
                 return f"Error: Agent output at index {i} is not a dictionary"
            
            if "agent_name" not in agent_output:
                return f"Error: 'agent_name' key is missing at index {i}"
            
            if "steps" not in agent_output:
               return f"Error: 'steps' key is missing at index {i}"
            
            if agent_output["steps"] is None:
               return f"Error: 'steps' data is None at index {i}"
            
            if not isinstance(agent_output["steps"], list):
               return f"Error: 'steps' data is not a list at index {i}"

            agent_name = agent_output["agent_name"]
            output += f"Run {(3-i)} (Agent: `{agent_name}`)\n\n"
            # output += "<details>\n<summary>Show/Hide Agent Steps</summary>\n\n"

            # Iterate over steps
            for j, step in enumerate(agent_output["steps"], start=3):
                 if not isinstance(step, dict):
                     return f"Error: step at index {j} is not a dictionary at {i} agent output."
                 
                 if step is None:
                     return f"Error: step at index {j} is None at {i} agent output"

                 if "role" not in step:
                     return f"Error: 'role' key missing at step {j} at {i} agent output."
                 
                 if "content" not in step:
                    return f"Error: 'content' key missing at step {j} at {i} agent output."
                 
                 if step["role"].strip() != "System:":  # Filter out system prompts
                    #  role = step["role"]
                     content = step["content"]
                     output += f"Step {(3-j)}:\n"
                     output += f"Response : {content}\n\n"

            # output += "</details>\n\n---\n"

        output += f"Overall Completion Time: `{overall_time}`"
        return output

   except json.JSONDecodeError as e :
        return f"Error during parsing json.JSONDecodeError : {e}"
   
   except Exception as e:
        return f"Error during parsing: {str(e)}"


def parse_spreadsheet_swarm_output(file_path: str) -> str:
    """Parses the SpreadSheetSwarm output CSV file and formats it for display."""
    if not file_path:
        return "Error: No file path provided for parsing."

    print(f"Parsing spreadsheet output from: {file_path}")

    try:
        with open(file_path, 'r', encoding='utf-8') as file:
            csv_reader = csv.reader(file)
            header = next(csv_reader, None)  # Read the header row
            if not header:
               return "Error: CSV file is empty or has no header"
            
            output = "### Spreadsheet Swarm Output ###\n\n"
            output += "| " + " | ".join(header) + " |\n" # Adding header
            output += "| " + " | ".join(["---"] * len(header)) + " |\n" # Adding header seperator

            for row in csv_reader:
                output += "| " + " | ".join(row) + " |\n" # Adding row
            
            output += "\n"
        return output
    
    except FileNotFoundError:
        return "Error: CSV file not found."
    except Exception as e:
        return f"Error during parsing CSV file: {str(e)}"
def parse_json_output(data:str) -> str:
    """Parses a JSON string and formats it for display."""
    if not data:
        return "Error: No data provided for parsing."

    print(f"Parsing json output from: {data}")
    try:
        parsed_data = json.loads(data)

        output = "### Swarm Metadata ###\n\n"
        
        for key,value in parsed_data.items():
            if key == "outputs":
              output += f"**{key}**:\n"
              if isinstance(value, list):
                for item in value:
                   output += f"  -  Agent Name : {item.get('agent_name', 'N/A')}\n"
                   output += f"     Task : {item.get('task', 'N/A')}\n"
                   output += f"     Result : {item.get('result', 'N/A')}\n"
                   output += f"     Timestamp : {item.get('timestamp', 'N/A')}\n\n"

              else :
                  output += f"  {value}\n"
            
            else :
                output += f"**{key}**: {value}\n"

        return output
    
    except json.JSONDecodeError as e:
        return f"Error: Invalid JSON format - {e}"
    
    except Exception as e:
        return f"Error during JSON parsing: {str(e)}"

class UI:
    def __init__(self, theme):
        self.theme = theme
        self.blocks = gr.Blocks(theme=self.theme)
        self.components = {}  # Dictionary to store UI components

    def create_markdown(self, text, is_header=False):
        if is_header:
            markdown = gr.Markdown(f"<h1 style='color: #ffffff; text-align: center;'>{text}</h1>")
        else:
            markdown = gr.Markdown(f"<p style='color: #cccccc; text-align: center;'>{text}</p>")
        self.components[f'markdown_{text}'] = markdown
        return markdown

    def create_text_input(self, label, lines=3, placeholder=""):
        text_input = gr.Textbox(
            label=label,
            lines=lines,
            placeholder=placeholder,
            elem_classes=["custom-input"],
        )
        self.components[f'text_input_{label}'] = text_input
        return text_input

    def create_slider(self, label, minimum=0, maximum=1, value=0.5, step=0.1):
        slider = gr.Slider(
            minimum=minimum,
            maximum=maximum,
            value=value,
            step=step,
            label=label,
            interactive=True,
        )
        self.components[f'slider_{label}'] = slider
        return slider

    def create_dropdown(self, label, choices, value=None, multiselect=False):
        if not choices:
            choices = ["No options available"]
        if value is None and choices:
            value = choices[0] if not multiselect else [choices[0]]

        dropdown = gr.Dropdown(
            label=label,
            choices=choices,
            value=value,
            interactive=True,
            multiselect=multiselect,
        )
        self.components[f'dropdown_{label}'] = dropdown
        return dropdown

    def create_button(self, text, variant="primary"):
        button = gr.Button(text, variant=variant)
        self.components[f'button_{text}'] = button
        return button

    def create_text_output(self, label, lines=10, placeholder=""):
        text_output = gr.Textbox(
            label=label,
            interactive=False,
            placeholder=placeholder,
            lines=lines,
            elem_classes=["custom-output"],
        )
        self.components[f'text_output_{label}'] = text_output
        return text_output

    def create_tab(self, label, content_function):
        with gr.Tab(label):
            content_function(self)

    def set_event_listener(self, button, function, inputs, outputs):
        button.click(function, inputs=inputs, outputs=outputs)

    def get_components(self, *keys):
        if not keys:
            return self.components  # return all components
        return [self.components[key] for key in keys]

    def create_json_output(self, label, placeholder=""):
        json_output = gr.JSON(
            label=label,
            value={},
            elem_classes=["custom-output"],
        )
        self.components[f'json_output_{label}'] = json_output
        return json_output

    def build(self):
        return self.blocks

    def create_conditional_input(self, component, visible_when, watch_component):
        """Create an input that's only visible under certain conditions"""
        watch_component.change(
            fn=lambda x: gr.update(visible=visible_when(x)),
            inputs=[watch_component],
            outputs=[component]
        )

    @staticmethod
    def create_ui_theme(primary_color="red"):
        return gr.themes.Ocean(
            primary_hue=primary_color,
            secondary_hue=primary_color,
            neutral_hue="gray",
        ).set(
            body_background_fill="#20252c",
            body_text_color="#f0f0f0",
            button_primary_background_fill=primary_color,
            button_primary_text_color="#ffffff",
            button_secondary_background_fill=primary_color,
            button_secondary_text_color="#ffffff",
            shadow_drop="0px 2px 4px rgba(0, 0, 0, 0.3)",
        )

    def create_agent_details_tab(self):
        """Create the agent details tab content."""
        with gr.Column():
            gr.Markdown("### Agent Details")
            gr.Markdown("""
            **Available Agent Types:**
            - Data Extraction Agent: Specialized in extracting relevant information
            - Summary Agent: Creates concise summaries of information
            - Analysis Agent: Performs detailed analysis of data

            **Swarm Types:**
            - ConcurrentWorkflow: Agents work in parallel
            - SequentialWorkflow: Agents work in sequence
            - AgentRearrange: Custom agent execution flow
            - MixtureOfAgents: Combines multiple agents with an aggregator
            - SpreadSheetSwarm: Specialized for spreadsheet operations
            - Auto: Automatically determines optimal workflow
            """)
            return gr.Column()

    def create_logs_tab(self):
        """Create the logs tab content."""
        with gr.Column():
            gr.Markdown("### Execution Logs")
            logs_display = gr.Textbox(
                label="System Logs",
                placeholder="Execution logs will appear here...",
                interactive=False,
                lines=10
            )
            return logs_display

def create_app():
    # Initialize UI
    theme = UI.create_ui_theme(primary_color="red")
    ui = UI(theme=theme)

    # Available providers and models
    providers = [
        "openai",
        "anthropic",
        "cohere",
        "gemini",
        "mistral",
        "groq",
        "nvidia_nim",
        "huggingface",
        "perplexity"
    ]

    filtered_models = {}

    for provider in providers:
        filtered_models[provider] = models_by_provider.get(provider, [])


    with ui.blocks:
        with gr.Row():
            with gr.Column(scale=4):  # Left column (80% width)
                ui.create_markdown("Swarms", is_header=True)
                ui.create_markdown(
                    "<b>The Enterprise-Grade Production-Ready Multi-Agent Orchestration Framework</b>"
                )
                with gr.Row():
                    with gr.Column(scale=4):
                        with gr.Row():
                            task_input = gr.Textbox(
                                label="Task Description",
                                placeholder="Describe your task here...",
                                lines=3
                            )
                        with gr.Row():
                            
                            with gr.Column(scale=1):
                                with gr.Row():
                            # Provider selection dropdown
                                    provider_dropdown = gr.Dropdown(
                                        label="Select Provider",
                                        choices=providers,
                                        value=providers[0] if providers else None,
                                        interactive=True,
                                    )
                                #  with gr.Row():
                                #     # Model selection dropdown (initially empty)
                                    model_dropdown = gr.Dropdown(
                                        label="Select Model",
                                        choices=[],
                                        interactive=True,
                                    )
                                with gr.Row():
                                    # API key input
                                    api_key_input = gr.Textbox(
                                        label="API Key",
                                        placeholder="Enter your API key",
                                        type="password",
                                    )
                            with gr.Column(scale=1):
                                with gr.Row():
                                    dynamic_slider = gr.Slider(
                                        label="Dyn. Temp",
                                        minimum=0,
                                        maximum=1,
                                        value=0.1,
                                        step=0.01
                                    )
                                        
                                #  with gr.Row():
                                #     max tokens slider
                                    max_loops_slider = gr.Slider(
                                        label="Max Loops",
                                        minimum=1,
                                        maximum=10,
                                        value=1,
                                        step=1
                                    )
                                    

                                with gr.Row():
                                    max_tokens_slider = gr.Slider(
                                        label="Max Tokens",
                                        minimum=100,
                                        maximum=10000,
                                        value=4000,
                                        step=100,
                                    )

                    with gr.Column(scale=2, min_width=200):
                         with gr.Column(scale=1):
                                # Get available agent prompts
                                available_prompts = list(AGENT_PROMPTS.keys()) if AGENT_PROMPTS else ["No agents available"]
                                agent_prompt_selector = gr.Dropdown(
                                    label="Select Agent Prompts",
                                    choices=available_prompts,
                                    value=[available_prompts[0]] if available_prompts else None,
                                    multiselect=True,
                                    interactive=True
                                )
                            # with gr.Column(scale=1):
                                # Get available swarm types
                                swarm_types = [
                                    "SequentialWorkflow", "ConcurrentWorkflow","AgentRearrange",
                                    "MixtureOfAgents", "SpreadSheetSwarm", "auto"
                                ]
                                agent_selector = gr.Dropdown(
                                    label="Select Swarm",
                                    choices=swarm_types,
                                    value=swarm_types[0],
                                    multiselect=False,
                                    interactive=True
                                )

                                # Flow configuration components for AgentRearrange
                                with gr.Column(visible=False) as flow_config:
                                    flow_text = gr.Textbox(
                                        label="Agent Flow Configuration",
                                        placeholder="Enter agent flow !",
                                        lines=2
                                    )
                                    gr.Markdown(
                                        """
                                        **Flow Configuration Help:**
                                        - Enter agent names separated by ' -> '
                                        - Example: Agent1 -> Agent2 -> Agent3
                                        - Use exact agent names from the prompts above
                                        """
                                    )
                    
                         
                            
                        #  with gr.Row():
                        #     temperature_slider = gr.Slider(
                        #         label="Temperature",
                        #         minimum=0,
                        #         maximum=1,
                        #         value=0.1,
                        #         step=0.01
                        #     )
                         
                         #Hidden textbox to store API Key
                         env_api_key_textbox = gr.Textbox(
                            value="",
                            visible=False
                        )

                with gr.Row():
                    with gr.Column(scale=1):
                        run_button = gr.Button("Run Task", variant="primary")
                        cancel_button = gr.Button("Cancel", variant="secondary")
                    with gr.Column(scale=1):
                        with gr.Row():
                            loading_status = gr.Textbox(
                                label="Status",
                                value="Ready",
                                interactive=False
                            )


                # Add loading indicator and status
                with gr.Row():
                    agent_output_display = gr.Textbox(
                        label="Agent Responses",
                        placeholder="Responses will appear here...",
                        interactive=False,
                        lines=10
                    )

                def update_flow_agents(agent_keys):
                    """Update flow agents based on selected agent prompts."""
                    if not agent_keys:
                       return [], "No agents selected"
                    agent_names = [key.split('.')[-1] for key in agent_keys]
                    return agent_names, "Select agents in execution order"

                def update_flow_preview(selected_flow_agents):
                    """Update flow preview based on selected agents."""
                    if not selected_flow_agents:
                        return "Flow will be shown here..."
                    flow = " -> ".join(selected_flow_agents)
                    return flow

                def update_ui_for_swarm_type(swarm_type):
                    """Update UI components based on selected swarm type."""
                    is_agent_rearrange = swarm_type == "AgentRearrange"
                    is_mixture = swarm_type == "MixtureOfAgents"
                    is_spreadsheet = swarm_type == "SpreadSheetSwarm"

                    max_loops = 5 if is_mixture or is_spreadsheet else 10

                    # Return visibility state for flow configuration and max loops update
                    return (
                        gr.update(visible=is_agent_rearrange),  # For flow_config
                        gr.update(maximum=max_loops),  # For max_loops_slider
                        f"Selected {swarm_type}"  # For loading_status
                    )
                def update_model_dropdown(provider):
                    """Update model dropdown based on selected provider."""
                    models = filtered_models.get(provider, [])
                    return gr.update(choices=models, value=models[0] if models else None)

                async def run_task_wrapper(task, max_loops, dynamic_temp, swarm_type, agent_prompt_selector, flow_text):
                    """Execute the task and update the UI with progress."""
                    try:
                        if not task:
                            yield "Please provide a task description.", "Error: Missing task"
                            return

                        if not agent_prompt_selector or len(agent_prompt_selector) == 0:
                            yield "Please select at least one agent.", "Error: No agents selected"
                            return

                        # Update status
                        yield "Processing...", "Running task..."

                        # Prepare flow for AgentRearrange
                        flow = None
                        if swarm_type == "AgentRearrange":
                            if not flow_text:
                                yield "Please provide the agent flow configuration.", "Error: Flow not configured"
                                return
                            flow = flow_text

                        # Execute task
                        result, router, error = await execute_task(
                            task=task,
                            max_loops=max_loops,
                            dynamic_temp=dynamic_temp, # THIS LINE IS CHANGED
                            swarm_type=swarm_type,
                            agent_keys=agent_prompt_selector,
                            flow=flow
                        )

                        if error:
                            yield f"Error: {error}", f"Error: {error}", ""
                            return

                        # Format output based on swarm type
                        output_lines = []
                        if swarm_type == "SpreadSheetSwarm":
                             # Check if the result is a file path or JSON
                             if os.path.exists(result):
                                parsed_output = parse_spreadsheet_swarm_output(result)
                                output_lines.append(parsed_output)
                             else:
                                parsed_output = parse_json_output(result)
                                output_lines.append(parsed_output)

                        elif swarm_type == "AgentRearrange":
                           parsed_output = parse_agent_rearrange_output(result)
                           output_lines.append(parsed_output)
                        elif swarm_type == "MixtureOfAgents":
                           parsed_output = parse_mixture_of_agents_output(result)
                           output_lines.append(parsed_output)
                        elif swarm_type == "SequentialWorkflow":
                           parsed_output = parse_sequential_workflow_output(result)
                           output_lines.append(parsed_output)
                        elif isinstance(result, dict):  # checking if result is dict or string.
                            
                            if swarm_type == "MixtureOfAgents":
                                # Add individual agent outputs
                                for key, value in result.items():
                                    if key != "Aggregated Summary":
                                        output_lines.append(f"### {key} ###\n{value}\n")
                                # Add aggregated summary at the end
                                if "Aggregated Summary" in result:
                                    output_lines.append(f"\n### Aggregated Summary ###\n{result['Aggregated Summary']}\n{'=' * 50}\n")
                            else:  # SequentialWorkflow, ConcurrentWorkflow, Auto
                                for key, value in result.items():
                                    output_lines.append(f"### {key} ###\n{value}\n{'=' * 50}\n")
                        elif isinstance(result, str):
                            output_lines.append(str(result))

                        yield "\n".join(output_lines), "Completed", api_key
                        
                    except Exception as e:
                         yield f"Error: {str(e)}", f"Error: {str(e)}", ""

                # Connect the update functions
                agent_selector.change(
                    fn=update_ui_for_swarm_type,
                    inputs=[agent_selector],
                    outputs=[flow_config, max_loops_slider, loading_status]
                )
                provider_dropdown.change(
                    fn=update_model_dropdown,
                    inputs=[provider_dropdown],
                    outputs=[model_dropdown]
                )

                # Create event trigger
                # Create event trigger for run button
                run_event = run_button.click(
                    fn=run_task_wrapper,
                    inputs=[
                        task_input,
                        max_loops_slider,
                        dynamic_slider,
                        agent_selector,
                        agent_prompt_selector,
                        flow_text,
                        provider_dropdown,
                        model_dropdown,
                        api_key_input,
                        max_tokens_slider
                    ],
                    outputs=[agent_output_display, loading_status, env_api_key_textbox]
                )

                # Connect cancel button to interrupt processing
                def cancel_task():
                    return "Task cancelled.", "Cancelled", ""

                cancel_button.click(
                    fn=cancel_task,
                    inputs=None,
                    outputs=[agent_output_display, loading_status, env_api_key_textbox],
                    cancels=run_event
                )

            with gr.Column(scale=1):  # Right column
                with gr.Tabs():
                    with gr.Tab("Agent Details"):
                        ui.create_agent_details_tab()

                    with gr.Tab("Logs"):
                        logs_display = ui.create_logs_tab()

                        def update_logs_display():
                            """Update logs display with current logs."""
                            return ""

                        # Update logs when tab is selected
                        logs_tab = gr.Tab("Logs")
                        logs_tab.select(fn=update_logs_display, inputs=None, outputs=[logs_display])

    return ui.build()

if __name__ == "__main__":
    app = create_app()
    app.launch()